import io
import os
import unittest.mock
from argparse import Namespace
from pathlib import PurePath

from play_helpers.ph_constants import PhConstants
from play_helpers.ph_util import PhUtil

STR_TEST_OBJ = 'test_obj :'


class test_obj_get_file_name_and_extn:
    def __init__(self, str_file_path, name_with_out_extn=None, only_extn=None, extn_with_out_dot=None, only_path=None,
                 ext_available=None, path_with_out_extn=None, only_folder_name=None,
                 expected_op=None):
        self.str_file_path = str_file_path
        self.name_with_out_extn = name_with_out_extn
        self.only_extn = only_extn
        self.extn_with_out_dot = extn_with_out_dot
        self.only_path = only_path
        self.ext_available = ext_available
        self.path_with_out_extn = path_with_out_extn
        self.only_folder_name = only_folder_name
        self.expected_op = expected_op


class test_obj_append_in_file_name:
    def __init__(self, str_file_path, str_append=None, sep=None, new_name=None, new_ext=None, file_path_is_dir=None,
                 ext_available_in_file_name=None, append_post=None, treat_folder_as_file=None, default_ext=None,
                 expected_op=None):
        self.str_file_path = str_file_path
        self.str_append = str_append
        self.sep = sep
        self.new_name = new_name
        self.new_ext = new_ext
        self.file_path_is_dir = file_path_is_dir
        self.ext_available_in_file_name = ext_available_in_file_name
        self.append_post = append_post
        self.treat_folder_as_file = treat_folder_as_file
        self.default_ext = default_ext
        self.expected_op = expected_op


class test_obj_get_version_from_name:
    def __init__(self, name, max_depth=None, expected_op=None):
        self.name = name
        self.max_depth = max_depth
        self.expected_op = expected_op


class test_obj_normalise_name_pandas_to_user:
    def __init__(self, name, all_caps_keywords=None, expected_op=None):
        self.name = name
        self.all_caps_keywords = all_caps_keywords
        self.expected_op = expected_op


class test_obj_dec_to_hex:
    def __init__(self, dec_num, digit_required=None, even_digits=None, assert_w_inbuilt=False, expected_op=None):
        self.dec_num = dec_num
        self.digit_required = digit_required
        self.even_digits = even_digits
        self.assert_w_inbuilt = assert_w_inbuilt
        self.expected_op = expected_op


class test_obj_hex_str_to_dec:
    def __init__(self, hex_str, signed_byte_handling=False, expected_op=None):
        self.hex_str = hex_str
        self.signed_byte_handling = signed_byte_handling
        self.expected_op = expected_op


class test_obj_to_hex_string:
    def __init__(self, hex_bytes, format=None, expected_op=None):
        self.hex_bytes = hex_bytes
        self.format = format
        self.expected_op = expected_op


class test_obj_hex_str_to_dec_list:
    def __init__(self, hex_str, signed_byte_handling=False, expected_op=None):
        self.hex_str = hex_str
        self.signed_byte_handling = signed_byte_handling
        self.expected_op = expected_op


class test_obj_rstrip_hex:
    def __init__(self, hex_data, expected_op=None):
        self.hex_data = hex_data
        self.expected_op = expected_op


class test_obj_analyse_data:
    def __init__(self, str_hex_data, cmt_to_print=None, print_also=None, expected_op=None):
        self.str_hex_data = str_hex_data
        self.cmt_to_print = cmt_to_print
        self.print_also = print_also
        self.expected_op = expected_op


class test_obj_print_iter:
    def __init__(self, the_iter, expected_op=None, header=None, list_as_str=None, sep=None):
        self.the_iter = the_iter
        self.expected_op = expected_op
        self.header = header
        self.list_as_str = list_as_str
        self.sep = sep


class test_obj_gen_isim_data:
    def __init__(self, imsi, mcc_mnc, pattern_or_data, expected_op=None):
        self.imsi = imsi
        self.mcc_mnc = mcc_mnc
        self.pattern_or_data = pattern_or_data
        self.expected_op = expected_op


class test_obj_print_seperator:
    def __init__(self, character, count, main_text, expected_op=None, tc_name=''):
        self.character = character
        self.count = count
        self.main_text = main_text
        self.expected_op = expected_op
        self.tc_name = tc_name


class test_obj_len_hex:
    def __init__(self, str_hex_data, output_in_str_format=None, expected_op=None):
        self.str_hex_data = str_hex_data
        self.output_in_str_format = output_in_str_format
        self.expected_op = expected_op


class util_test(unittest.TestCase):

    def test_get_file_name_and_extn(self):
        """
        :return:
        """
        src_file_path_bs = r'D:\\Other\\sgp22\\sgp22.py'
        src_file_path_fs = r'D:/Other/sgp22/sgp22.py'
        op_file_name = 'sgp22.py'
        op_folder_path = r'D:/Other/sgp22/'
        op_folder_name = 'sgp22'
        test_obj_pool = [
            #
            test_obj_get_file_name_and_extn(str_file_path=r'..\\..\\UserData\\Temp\\', expected_op='Temp'),
            test_obj_get_file_name_and_extn(str_file_path=r'..\\..\\UserData\\Temp', expected_op='Temp'),
            test_obj_get_file_name_and_extn(str_file_path=src_file_path_bs, expected_op=op_file_name),
            test_obj_get_file_name_and_extn(str_file_path=src_file_path_fs, expected_op=op_file_name),
            #
            test_obj_get_file_name_and_extn(str_file_path=src_file_path_bs, only_path=True, expected_op=op_folder_path),
            test_obj_get_file_name_and_extn(str_file_path=src_file_path_fs, only_path=True, expected_op=op_folder_path),
            #
            test_obj_get_file_name_and_extn(str_file_path=src_file_path_bs, only_folder_name=True,
                                            expected_op=op_folder_name),
            test_obj_get_file_name_and_extn(str_file_path=src_file_path_fs, only_folder_name=True,
                                            expected_op=op_folder_name),
            #
            test_obj_get_file_name_and_extn(str_file_path=r'D:/Other/sgp22', only_path=True, expected_op=r'D:/Other/'),
            test_obj_get_file_name_and_extn(str_file_path=r'D:/Other/sgp22/', only_path=True, expected_op=r'D:/Other/'),
        ]
        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                self.assertEqual(
                    PhUtil.get_file_name_and_extn(
                        file_path=test_obj.str_file_path,
                        name_with_out_extn=test_obj.name_with_out_extn,
                        only_extn=test_obj.only_extn,
                        extn_with_out_dot=test_obj.extn_with_out_dot,
                        only_path=test_obj.only_path,
                        ext_available=test_obj.ext_available,
                        path_with_out_extn=test_obj.path_with_out_extn,
                        only_folder_name=test_obj.only_folder_name
                    ),
                    test_obj.expected_op)

    def test_appendInFileName(self):
        """
        :return:
        """
        op_file_name = 'myNameIsKhan.tmp'
        op_file_path = os.sep.join([PhUtil.path_default_out_folder, 'Youtube_PlayList.tmp'])
        op_file_path_dir = 'D:\\Other\\Github_Self\\playhelpers\\play_helpers\\test\\out_Youtube_PlayList\\.tmp'
        src_file_path = 'D:/abc/def/123/out_file_12564.txt'
        default_ext = '.default'
        test_obj_pool = [
            test_obj_append_in_file_name(str_file_path='D:\\Other\\python_gen\\sgp22\\sgp22.py', str_append='mapping',
                                         expected_op='D:\\Other\\python_gen\\sgp22\\sgp22_mapping.py'),
            test_obj_append_in_file_name(str_file_path='D:\\Other\\python_gen\\sgp22\\sgp22.py.py', new_ext='.pyc',
                                         expected_op='D:\\Other\\python_gen\\sgp22\\sgp22.py.pyc'),
            test_obj_append_in_file_name(str_file_path='temp_file.txt', str_append='pjkl',
                                         expected_op='temp_file_pjkl.txt'),
            test_obj_append_in_file_name(str_file_path=r'F:\Android\obb\com.jio.yt\temp_file.txt', str_append='pjkl',
                                         expected_op=r'F:\Android\obb\com.jio.yt\temp_file_pjkl.txt'),
            test_obj_append_in_file_name(str_file_path='temp_file.txt', str_append='pjkl', append_post=False,
                                         expected_op='pjkl_temp_file.txt'),
            test_obj_append_in_file_name(str_file_path=PhUtil.get_file_name_and_extn(
                r'F:\Android\data\com.jio.myhome.mediasharedmp\files\uploads\myNameIsKhan.txt'),
                new_ext='.tmp', expected_op=op_file_name),
            test_obj_append_in_file_name(str_file_path=PhUtil.get_file_name_and_extn(
                'F://Android//data//com.jio.myhome.mediasharedmp//files//uploads/myNameIsKhan.txt'),
                new_ext='.tmp', expected_op=op_file_name),
            test_obj_append_in_file_name(str_file_path=PhUtil.get_file_name_and_extn('myNameIsKhan.txt'),
                                         new_ext='.tmp', expected_op=op_file_name),
            test_obj_append_in_file_name(str_file_path='myNameIsKhan.txt', new_ext='.tmp', expected_op=op_file_name),
            test_obj_append_in_file_name(str_file_path='myNameIsKhan', new_ext='.tmp', expected_op=op_file_name),
            test_obj_append_in_file_name(str_file_path='target_profile', str_append=str(5 + 1),
                                         expected_op='target_profile_6'),
            test_obj_append_in_file_name(str_file_path='', str_append='pjkl', append_post=False, expected_op='pjkl'),
            test_obj_append_in_file_name(str_file_path='', str_append='pjkl', new_ext='.log', expected_op='pjkl.log'),
            test_obj_append_in_file_name(str_file_path='', str_append='', new_ext='.temp', expected_op='.temp'),
            test_obj_append_in_file_name(str_file_path='', str_append='', expected_op=''),
            test_obj_append_in_file_name(str_file_path='pjklmnop.txt', str_append=['with', 'text'],
                                         expected_op='pjklmnop_with_text.txt'),
            test_obj_append_in_file_name(str_file_path='pjklmnop.txt', str_append=['output', 'data', '20210721'],
                                         new_ext='.out', expected_op='pjklmnop_output_data_20210721.out'),
            test_obj_append_in_file_name(
                str_file_path=os.sep.join([PhUtil.path_default_tst_folder, 'prog', 'raw', 'prog']), new_ext='.txt',
                expected_op=os.sep.join([PhUtil.path_default_tst_folder, 'prog', 'raw', 'prog.txt'])),
            test_obj_append_in_file_name(
                str_file_path=os.sep.join([PhUtil.path_default_out_folder, 'names_Generated']),
                str_append='fileNameKeyword', new_ext='.txt',
                expected_op=os.sep.join([PhUtil.path_default_out_folder, 'names_Generated_fileNameKeyword.txt'])),
            test_obj_append_in_file_name(
                str_file_path=os.sep.join([PhUtil.path_default_out_folder, 'names_Generated.asn']),
                str_append='fileNameKeyword', new_ext='.txt',
                expected_op=os.sep.join([PhUtil.path_default_out_folder, 'names_Generated_fileNameKeyword.txt'])),
            test_obj_append_in_file_name(
                str_file_path='names_Generated.asn', str_append='fileNameKeyword', new_ext='.txt',
                expected_op='names_Generated_fileNameKeyword.txt'),
            test_obj_append_in_file_name(str_file_path='PROFILE_OPERATIONAL1.8.35.txt', str_append='1',
                                         expected_op='PROFILE_OPERATIONAL1.8.35_1.txt'),
            test_obj_append_in_file_name(str_file_path='PROFILE_OPERATIONAL1.8.35.txt', str_append='1',
                                         sep='.', expected_op='PROFILE_OPERATIONAL1.8.35.1.txt'),
            test_obj_append_in_file_name(str_file_path='profile_name', str_append=['profile_data', 'extra_keyword'],
                                         new_ext='.txt', ext_available_in_file_name=False,
                                         expected_op='profile_name_profile_data_extra_keyword.txt'),
            test_obj_append_in_file_name(str_file_path='PROFILE_OPERATIONAL1.8.35',
                                         str_append=['profile_data'],
                                         new_ext='.txt', ext_available_in_file_name=False,
                                         expected_op='PROFILE_OPERATIONAL1.8.35_profile_data.txt'),
            test_obj_append_in_file_name(str_file_path='PROFILE_OPERATIONAL1.8.35',
                                         str_append=['profile_data'],
                                         new_ext='.txt', expected_op='PROFILE_OPERATIONAL1.8_profile_data.txt'),
            test_obj_append_in_file_name(str_file_path=src_file_path, new_ext='.out', new_name='123456',
                                         expected_op='D:/abc/def/123/123456.out'),
            test_obj_append_in_file_name(str_file_path=src_file_path, new_name='123456',
                                         expected_op='D:/abc/def/123/123456.txt'),
            test_obj_append_in_file_name(
                str_file_path=PhUtil.get_file_name_and_extn(src_file_path,
                                                            only_path=True) + os.sep + '_file_name', new_ext='.asn',
                expected_op='D:/abc/def/123/\_file_name.asn'),
            test_obj_append_in_file_name(str_file_path=os.sep.join(
                [PhUtil.get_file_name_and_extn(src_file_path, only_path=True),
                 '_file_name']), new_ext='.asn', expected_op='D:/abc/def/123/\_file_name.asn'),
            test_obj_append_in_file_name(str_file_path=src_file_path, str_append=PurePath(src_file_path).parent.name,
                                         new_ext='.asn',
                                         expected_op='D:/abc/def/123/out_file_12564_123.asn'),
            test_obj_append_in_file_name(
                str_file_path=PhUtil.path_default_out_folder, str_append=['Youtube', 'PlayList'], new_ext='.tmp',
                expected_op=PhUtil.path_default_out_folder + '_Youtube_PlayList.tmp'),
            test_obj_append_in_file_name(str_file_path='.txt', str_append='mapping',
                                         expected_op='mapping.txt'),
            test_obj_append_in_file_name(str_file_path='D:\\Other\\python_gen\\sgp22\\sgp22.py', str_append='mapping',
                                         file_path_is_dir=True,
                                         expected_op='D:\\Other\\python_gen\\sgp22\\sgp22.py\\mapping'),
            test_obj_append_in_file_name(str_file_path='D:\\Other\\python_gen\\sgp22\\sgp22.py', str_append='mapping',
                                         file_path_is_dir=True, treat_folder_as_file=True,
                                         expected_op='D:\\Other\\python_gen\\sgp22\\sgp22.py_mapping'),
            test_obj_append_in_file_name(str_file_path='D:\\Other\\python_gen\\sgp22\\sgp22', str_append='mapping',
                                         file_path_is_dir=True,
                                         expected_op='D:\\Other\\python_gen\\sgp22\\sgp22\\mapping'),
            test_obj_append_in_file_name(str_file_path='D:\\Other\\python_gen\\sgp22\\sgp22', str_append='mapping',
                                         file_path_is_dir=True, treat_folder_as_file=True,
                                         expected_op='D:\\Other\\python_gen\\sgp22\\sgp22_mapping'),
            test_obj_append_in_file_name(
                str_file_path=PhUtil.path_default_out_folder + os.sep, str_append=['Youtube', 'PlayList'],
                new_ext='.tmp', file_path_is_dir=False, expected_op=op_file_path),
            test_obj_append_in_file_name(
                str_file_path=PhUtil.path_default_out_folder + os.sep, str_append=['Youtube', 'PlayList'],
                new_ext='.tmp', file_path_is_dir=True, expected_op=op_file_path),
            test_obj_append_in_file_name(
                str_file_path=PhUtil.path_default_out_folder + os.sep, str_append=['Youtube', 'PlayList'],
                new_ext='.tmp', file_path_is_dir=True, treat_folder_as_file=True, expected_op=op_file_path_dir),
            test_obj_append_in_file_name(
                str_file_path=os.sep.join([PhUtil.path_default_out_folder, '']), str_append=['Youtube', 'PlayList'],
                new_ext='.tmp', expected_op=op_file_path),
            test_obj_append_in_file_name(
                str_file_path=os.sep.join([PhUtil.path_default_out_folder, '']), new_name='dest_vs_block',
                new_ext='.txt', expected_op=os.sep.join([PhUtil.path_default_out_folder, 'dest_vs_block.txt'])),
            test_obj_append_in_file_name(str_file_path='\\sgp22\\sgp22.py', str_append='mapping',
                                         expected_op='\\sgp22\\sgp22_mapping.py', default_ext=default_ext),
            test_obj_append_in_file_name(str_file_path='\\sgp22\\sgp22.py', str_append='mapping', new_ext='.pyc',
                                         expected_op='\\sgp22\\sgp22_mapping.pyc', default_ext=default_ext),
            test_obj_append_in_file_name(str_file_path='\\sgp22\\sgp22', str_append='mapping',
                                         expected_op='\\sgp22\\sgp22_mapping.default', default_ext=default_ext),
        ]
        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                self.assertEqual(
                    PhUtil.append_in_file_name(str_file_path=test_obj.str_file_path, str_append=test_obj.str_append,
                                               sep=test_obj.sep, new_name=test_obj.new_name, new_ext=test_obj.new_ext,
                                               file_path_is_dir=test_obj.file_path_is_dir,
                                               ext_available_in_file_name=test_obj.ext_available_in_file_name,
                                               treat_folder_as_file=test_obj.treat_folder_as_file,
                                               default_ext=test_obj.default_ext,
                                               append_post=test_obj.append_post), test_obj.expected_op)

    def test_get_version_from_name(self):
        """
        :return:
        """
        test_obj_pool = [
            test_obj_get_version_from_name(name='SGP.22-v2.0.pdf', max_depth=None, expected_op='v2_0'),
            test_obj_get_version_from_name(name='SGP.22-v2.0.txt', max_depth=None, expected_op='v2_0'),
            test_obj_get_version_from_name(name='SGP.22-v2.2.1-2.txt', max_depth=None, expected_op='v2_2_1_2'),
            test_obj_get_version_from_name(name='SGP.22-v2.2.1-2.txt', max_depth=3, expected_op='v2_2_1'),
            test_obj_get_version_from_name(name='SGP.22-v2.2.2.txt', max_depth=None, expected_op='v2_2_2'),
            test_obj_get_version_from_name(name='SGP.22_v1.0.txt', max_depth=None, expected_op='v1_0'),
            test_obj_get_version_from_name(name='SGP.22_v2.2.txt', max_depth=None, expected_op='v2_2'),
            test_obj_get_version_from_name(name='SGP.22_v2.0-Technical_Specification-eSIMRSP.pdf', max_depth=None,
                                           expected_op='v2_0'),
            test_obj_get_version_from_name(name='SGP.22_v2.0-Technical Specification-eSIMRSP.pdf', max_depth=None,
                                           expected_op='v2_0'),
        ]
        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                self.assertEqual(
                    PhUtil.get_version_from_name(test_obj.name, max_depth=test_obj.max_depth),
                    test_obj.expected_op)

    def test_normalise_name_pandas_to_user(self):

        """
        :return:
        """

        test_obj_pool = [
            test_obj_normalise_name_pandas_to_user(name='ICCID', expected_op='ICCID'),
            test_obj_normalise_name_pandas_to_user(name='IMSI', expected_op='IMSI'),
            test_obj_normalise_name_pandas_to_user(name='MATCHING_ID', expected_op='Matching ID'),
            test_obj_normalise_name_pandas_to_user(name='MSISDN', expected_op='MSISDN'),
            test_obj_normalise_name_pandas_to_user(name='CONFIRMATION_CODE', all_caps_keywords=['CODE'],
                                                   expected_op='Confirmation CODE'),
            test_obj_normalise_name_pandas_to_user(name='CONFIRMATION_CODE', all_caps_keywords=['CONFIRMATION', 'CODE'],
                                                   expected_op='CONFIRMATION CODE'),
            test_obj_normalise_name_pandas_to_user(name='CONFIRMATION_CODE', expected_op='Confirmation Code'),
        ]
        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                self.assertEqual(PhUtil.normalise_name_pandas_to_user(test_obj.name, test_obj.all_caps_keywords),
                                 test_obj.expected_op)

    def test_dec_to_hex(self):
        """

        :return:
        """
        test_obj_pool = [
            test_obj_dec_to_hex(15, expected_op='0F'),
            test_obj_dec_to_hex(15, expected_op='0F', even_digits=True),
            test_obj_dec_to_hex(15, expected_op='F', even_digits=False),
            test_obj_dec_to_hex(16, expected_op='10'),
            test_obj_dec_to_hex(16, expected_op='10', even_digits=True),
            test_obj_dec_to_hex(16, expected_op='10', even_digits=False),
            test_obj_dec_to_hex(15, digit_required=32, expected_op='0000000000000000000000000000000F'),
            test_obj_dec_to_hex(16, digit_required=32, expected_op='00000000000000000000000000000010'),
            test_obj_dec_to_hex(15, expected_op='0xf', assert_w_inbuilt=True),
            test_obj_dec_to_hex(16, expected_op='0x10', assert_w_inbuilt=True),
            test_obj_dec_to_hex(-68, expected_op='BC'),
            test_obj_dec_to_hex([10, -68, -46, 85], expected_op='0ABCD255'),
        ]

        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                if test_obj.assert_w_inbuilt:
                    self.assertEqual(hex(test_obj.dec_num), test_obj.expected_op)
                    continue
                if test_obj.digit_required is not None:
                    self.assertEqual(PhUtil.dec_to_hex(test_obj.dec_num, test_obj.digit_required),
                                     test_obj.expected_op)
                else:
                    if test_obj.even_digits is not None:
                        self.assertEqual(PhUtil.dec_to_hex(test_obj.dec_num, test_obj.digit_required,
                                                           even_digits=test_obj.even_digits),
                                         test_obj.expected_op)
                    else:
                        self.assertEqual(PhUtil.dec_to_hex(test_obj.dec_num), test_obj.expected_op)

    def test_hex_str_to_dec(self):
        """

        :return:
        """
        test_obj_pool = [
            test_obj_hex_str_to_dec('fff', expected_op=4095),
            test_obj_hex_str_to_dec('0fff', expected_op=4095),
            test_obj_hex_str_to_dec('FFF', expected_op=4095),
            test_obj_hex_str_to_dec('0xfff', expected_op=4095),
            test_obj_hex_str_to_dec('10', expected_op=16),
            test_obj_hex_str_to_dec('0x10', expected_op=16),
            test_obj_hex_str_to_dec('0x0A', expected_op=10),
            test_obj_hex_str_to_dec('0xBC', signed_byte_handling=True, expected_op=-68),
            test_obj_hex_str_to_dec('0xBC', signed_byte_handling=False, expected_op=188),
        ]

        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                self.assertEqual(
                    PhUtil.hex_str_to_dec(test_obj.hex_str, signed_byte_handling=test_obj.signed_byte_handling),
                    test_obj.expected_op)

    def test_to_hex_string(self):
        """

        :return:
        """
        hex_bytes = [0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03]
        test_obj_pool = [
            test_obj_to_hex_string(hex_bytes, expected_op='3B 65 00 00 9C 11 01 01 03'),
            test_obj_to_hex_string(hex_bytes, format=PhConstants.FORMAT_HEX_STRING_AS_COMMA,
                                   expected_op='3B, 65, 00, 00, 9C, 11, 01, 01, 03'),
            test_obj_to_hex_string(hex_bytes, format=PhConstants.FORMAT_HEX_STRING_AS_HEX,
                                   expected_op='0x3B 0x65 0x00 0x00 0x9C 0x11 0x01 0x01 0x03'),
            test_obj_to_hex_string(hex_bytes,
                                   format=PhConstants.FORMAT_HEX_STRING_AS_HEX | PhConstants.FORMAT_HEX_STRING_AS_COMMA,
                                   expected_op='0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03'),
            test_obj_to_hex_string(hex_bytes,
                                   format=PhConstants.FORMAT_HEX_STRING_AS_HEX | PhConstants.FORMAT_HEX_STRING_AS_UPPERCASE,
                                   expected_op='0X3B 0X65 0X00 0X00 0X9C 0X11 0X01 0X01 0X03'),
            test_obj_to_hex_string(hex_bytes,
                                   format=PhConstants.FORMAT_HEX_STRING_AS_HEX | PhConstants.FORMAT_HEX_STRING_AS_UPPERCASE | PhConstants.FORMAT_HEX_STRING_AS_COMMA,
                                   expected_op='0X3B, 0X65, 0X00, 0X00, 0X9C, 0X11, 0X01, 0X01, 0X03'),
            test_obj_to_hex_string(hex_bytes, format=PhConstants.FORMAT_HEX_STRING_AS_PACK,
                                   expected_op='3B6500009C11010103'),
        ]

        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                if test_obj.format:
                    self.assertEqual(PhUtil.to_hex_string(test_obj.hex_bytes, test_obj.format), test_obj.expected_op)
                else:
                    self.assertEqual(PhUtil.to_hex_string(test_obj.hex_bytes), test_obj.expected_op)

    def test_hex_str_to_dec_list(self):
        """

        :return:
        """
        test_obj_pool = [
            test_obj_hex_str_to_dec_list('A000000559 1010 FFFFFF FF89 000001 0',
                                         expected_op=[160, 0, 0, 5, 89, 16, 16, 255, 255, 255, 255, 137, 0, 0, 1, 0]),
            test_obj_hex_str_to_dec_list('A000000559 1010 FFFFFF FF89 000001 00', signed_byte_handling=False,
                                         expected_op=[160, 0, 0, 5, 89, 16, 16, 255, 255, 255, 255, 137, 0, 0, 1, 0]),
            test_obj_hex_str_to_dec_list('A000000559 1010 FFFFFF FF89 000001 00', signed_byte_handling=True,
                                         expected_op=[-96, 0, 0, 5, 89, 16, 16, -1, -1, -1, -1, -119, 0, 0, 1, 0]),
        ]

        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                self.assertEqual(PhUtil.hex_str_to_dec_list(test_obj.hex_str,
                                                            signed_byte_handling=test_obj.signed_byte_handling),
                                 test_obj.expected_op)

    def test_rstrip_hex(self):
        """

        :return:
        """
        test_obj_pool = [
            test_obj_rstrip_hex('37F08037F010FFFFFFFFFFFF', expected_op='37F08037F010'),
            test_obj_rstrip_hex('37F08037F010FFFFFFFFFFF', expected_op='37F08037F010F'),
            test_obj_rstrip_hex(
                '37F030800037F020800037F030008037F0200080270243808017F671808027F420808027F430808027F44080801300148080371211808037F630808012F440808047F810808047F020808022F210808002F802808032F4028080330450808017F410808035F050808047F440808017F210808002F602808007F420808005F281808015F0988080FFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0F',
                expected_op='37F030800037F020800037F030008037F0200080270243808017F671808027F420808027F430808027F44080801300148080371211808037F630808012F440808047F810808047F020808022F210808002F802808032F4028080330450808017F410808035F050808047F440808017F210808002F602808007F420808005F281808015F0988080FFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0FFFFFFF3F0F'),
            test_obj_rstrip_hex('61184F10A0000000871002FF33FF01890000010050045553494DFFFFFFFFFFFFFFFFFFFFFFFF',
                                expected_op='61184F10A0000000871002FF33FF01890000010050045553494D'),
        ]

        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                self.assertEqual(PhUtil.rstrip_hex_str(test_obj.hex_data), test_obj.expected_op)

    def test_analyse_data(self):
        """

        :return:
        """
        test_obj_pool = [
            test_obj_analyse_data('123456', expected_op='Length: 6 digits(s) / 03 byte(s), Data: 123456'),
            test_obj_analyse_data('12345', expected_op='Odd Length, Length: 5 digits(s) / 02 byte(s), Data: 12345'),
            test_obj_analyse_data('123456', cmt_to_print='1st',
                                  expected_op='\n1st\t:Length: 6 digits(s) / 03 byte(s), Data: 123456'),
            test_obj_analyse_data('12345', cmt_to_print='2nd',
                                  expected_op='\n2nd\t:Odd Length, Length: 5 digits(s) / 02 byte(s), Data: 12345'),
            test_obj_analyse_data('123456', cmt_to_print='1st', print_also=True,
                                  expected_op='\n1st\t:Length: 6 digits(s) / 03 byte(s), Data: 123456'),
            test_obj_analyse_data('12345', cmt_to_print='2nd', print_also=True,
                                  expected_op='\n2nd\t:Odd Length, Length: 5 digits(s) / 02 byte(s), Data: 12345'),
            test_obj_analyse_data('0123456789', cmt_to_print='3rd', print_also=True,
                                  expected_op='\n3rd\t:Length: 10 digits(s) / 05 byte(s), Data: 0123456789'),
            test_obj_analyse_data('01234567890', cmt_to_print='3rd', print_also=True,
                                  expected_op='\n3rd\t:Odd Length, Length: 11 digits(s) / 05 byte(s), Data: 01234567890'),
            test_obj_analyse_data('01234567890123456789', cmt_to_print='4th', print_also=True,
                                  expected_op='\n4th\t:Length: 20 digits(s) / 0A byte(s), Data: 01234567890123456789'),
            test_obj_analyse_data('012345678901234567890', cmt_to_print='4th', print_also=True,
                                  expected_op='\n4th\t:Odd Length, Length: 21 digits(s) / 0A byte(s), Data: 012345678901234567890'),
        ]

        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                if test_obj.cmt_to_print and test_obj.print_also:
                    self.assertEqual(PhUtil.analyse_data(test_obj.str_hex_data, test_obj.cmt_to_print,
                                                         test_obj.print_also), test_obj.expected_op)
                elif test_obj.cmt_to_print:
                    self.assertEqual(PhUtil.analyse_data(test_obj.str_hex_data, test_obj.cmt_to_print),
                                     test_obj.expected_op)
                else:
                    self.assertEqual(PhUtil.analyse_data(test_obj.str_hex_data), test_obj.expected_op)

    def test_print_iter(self):
        dict_1 = {'pre_defined_script': 'cmds_esim_ara_m_select',
                  'pre_defined_script_multi': None,
                  'custom_script': '',
                  'iccid': '89111100000000000008',
                  'imsi': '222013000000000',
                  'aid_usim': 'A0000000871002FF33FF018900000100',
                  'aid_isim': 'A0000000871004FF33FF018900000100',
                  'param_gen_1': None,
                  'param_gen_2': '',
                  'auto_fetch': 'True',
                  'auto_get_response': 'True',
                  'auto_cmd_for_6cxx': 'True',
                  'log_mode_append': 'True',
                  'help_mode': 'False',
                  'virtual_card': 'False'}
        dict_1_print = f'pre_defined_script: cmds_esim_ara_m_select\n' \
                       f'pre_defined_script_multi: None\n' \
                       f'custom_script: \n' \
                       f'iccid: 89111100000000000008\n' \
                       f'imsi: 222013000000000\n' \
                       f'aid_usim: A0000000871002FF33FF018900000100\n' \
                       f'aid_isim: A0000000871004FF33FF018900000100\n' \
                       f'param_gen_1: None\n' \
                       f'param_gen_2: \n' \
                       f'auto_fetch: True\n' \
                       f'auto_get_response: True\n' \
                       f'auto_cmd_for_6cxx: True\n' \
                       f'log_mode_append: True\n' \
                       f'help_mode: False\n' \
                       f'virtual_card: False\n' \
                       f'\n'
        dict_1_print_sep = f'pre_defined_script: cmds_esim_ara_m_select; ' \
                           f'pre_defined_script_multi: None; ' \
                           f'custom_script: ; ' \
                           f'iccid: 89111100000000000008; ' \
                           f'imsi: 222013000000000; ' \
                           f'aid_usim: A0000000871002FF33FF018900000100; ' \
                           f'aid_isim: A0000000871004FF33FF018900000100; ' \
                           f'param_gen_1: None; ' \
                           f'param_gen_2: ; ' \
                           f'auto_fetch: True; ' \
                           f'auto_get_response: True; ' \
                           f'auto_cmd_for_6cxx: True; ' \
                           f'log_mode_append: True; ' \
                           f'help_mode: False; ' \
                           f'virtual_card: False' \
                           f'\n'
        list_1 = ['F9526EFF9CAE23D00C77860F9BAEB301', '554B60ACA30428BF920918F16D102218',
                  '237CC88B687C5C97F9FD9A504BD7BB4C', '408FE089A8665EFC3419EF20277C2020',
                  '4DD24459B14E1E8A71C17578854A94EB', '3A0FD2809DAE5A7E884D0B22FFFA6134',
                  'FEF2A8D1E05CF65B4A61CC3BF05F5395', '4DD7419D2AFB3D44DC19BF0D344F36AC',
                  'D241FD541C09A30E87429E42CB8C66BA', '22FB24FF6D553A0A5589B171A2EAB725']
        list_1_print = f'F9526EFF9CAE23D00C77860F9BAEB301\n' \
                       f'554B60ACA30428BF920918F16D102218\n' \
                       f'237CC88B687C5C97F9FD9A504BD7BB4C\n' \
                       f'408FE089A8665EFC3419EF20277C2020\n' \
                       f'4DD24459B14E1E8A71C17578854A94EB\n' \
                       f'3A0FD2809DAE5A7E884D0B22FFFA6134\n' \
                       f'FEF2A8D1E05CF65B4A61CC3BF05F5395\n' \
                       f'4DD7419D2AFB3D44DC19BF0D344F36AC\n' \
                       f'D241FD541C09A30E87429E42CB8C66BA\n' \
                       f'22FB24FF6D553A0A5589B171A2EAB725\n' \
                       f'\n'
        list_1_print_sep = f'F9526EFF9CAE23D00C77860F9BAEB301; ' \
                           f'554B60ACA30428BF920918F16D102218; ' \
                           f'237CC88B687C5C97F9FD9A504BD7BB4C; ' \
                           f'408FE089A8665EFC3419EF20277C2020; ' \
                           f'4DD24459B14E1E8A71C17578854A94EB; ' \
                           f'3A0FD2809DAE5A7E884D0B22FFFA6134; ' \
                           f'FEF2A8D1E05CF65B4A61CC3BF05F5395; ' \
                           f'4DD7419D2AFB3D44DC19BF0D344F36AC; ' \
                           f'D241FD541C09A30E87429E42CB8C66BA; ' \
                           f'22FB24FF6D553A0A5589B171A2EAB725' \
                           f'\n'
        list_1_str_print = f"[" \
                           f"'F9526EFF9CAE23D00C77860F9BAEB301', " \
                           f"'554B60ACA30428BF920918F16D102218', " \
                           f"'237CC88B687C5C97F9FD9A504BD7BB4C', " \
                           f"'408FE089A8665EFC3419EF20277C2020', " \
                           f"'4DD24459B14E1E8A71C17578854A94EB', " \
                           f"'3A0FD2809DAE5A7E884D0B22FFFA6134', " \
                           f"'FEF2A8D1E05CF65B4A61CC3BF05F5395', " \
                           f"'4DD7419D2AFB3D44DC19BF0D344F36AC', " \
                           f"'D241FD541C09A30E87429E42CB8C66BA', " \
                           f"'22FB24FF6D553A0A5589B171A2EAB725'" \
                           f"]" \
                           f'\n'
        list_1_str_print_header = 'List as Str'
        header_joiner = ': '
        test_obj_pool = [
            test_obj_print_iter('37F08037F010FFFFFFFFFFFF', expected_op='37F08037F010FFFFFFFFFFFF\n'),
            test_obj_print_iter(123, expected_op='123\n'),
            test_obj_print_iter(123.09, expected_op='123.09\n'),
            test_obj_print_iter(dict_1, expected_op=dict_1_print),
            test_obj_print_iter(Namespace(**dict_1), expected_op=dict_1_print),
            test_obj_print_iter(list_1, expected_op=list_1_print),
            test_obj_print_iter(list_1, expected_op=list_1_str_print, list_as_str=True),
            test_obj_print_iter(list_1, expected_op=header_joiner.join([list_1_str_print_header, list_1_str_print]),
                                header=list_1_str_print_header, list_as_str=True),
            test_obj_print_iter(list_1, expected_op=list_1_print_sep, sep=PhConstants.SEPERATOR_MULTI_OBJ),
            test_obj_print_iter(dict_1, expected_op=dict_1_print_sep, sep=PhConstants.SEPERATOR_MULTI_OBJ),
        ]

        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                self.assert_stdout_print_iter(test_obj)

    @unittest.mock.patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout_print_iter(self, test_obj, mock_stdout):
        self.maxDiff = None
        PhUtil.print_iter(the_iter=test_obj.the_iter, list_as_str=test_obj.list_as_str, header=test_obj.header,
                          sep=test_obj.sep)
        actual_value = mock_stdout.getvalue()
        expected_value = test_obj.expected_op
        self.assertEqual(actual_value, expected_value)

    def test_len_hex(self):
        """
        :return:
        """
        test_obj_pool = [
            test_obj_len_hex(str_hex_data='123456', output_in_str_format=True, expected_op='03'),
            test_obj_len_hex(str_hex_data='123456', output_in_str_format=False, expected_op=3),
            test_obj_len_hex(str_hex_data='123456', expected_op=3),
            test_obj_len_hex(str_hex_data='01234567890123456789', output_in_str_format=True, expected_op='0A'),
            test_obj_len_hex(str_hex_data='01234567890123456789', output_in_str_format=False, expected_op=0x0A),
            test_obj_len_hex(str_hex_data='01234567890123456789', expected_op=0x0A),

        ]
        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                if test_obj.output_in_str_format:
                    self.assertEqual(
                        PhUtil.len_hex(str_hex_data=test_obj.str_hex_data,
                                       output_in_str_format=test_obj.output_in_str_format),
                        test_obj.expected_op)
                else:
                    self.assertEqual(
                        PhUtil.len_hex(str_hex_data=test_obj.str_hex_data),
                        test_obj.expected_op)

    def test_gen_isim_data(self):
        """
        :return:
        """
        test_obj_pool = [
            test_obj_gen_isim_data(imsi='234507095401016', mcc_mnc='23450',
                                   pattern_or_data='ims.mnc${MNC}.mcc${MCC}.3gppnetwork.org',
                                   expected_op='ims.mnc050.mcc234.3gppnetwork.org'),
            test_obj_gen_isim_data(imsi='234507095401016', mcc_mnc='234507',
                                   pattern_or_data='ims.mnc${MNC}.mcc${MCC}.3gppnetwork.org',
                                   expected_op='ims.mnc507.mcc234.3gppnetwork.org'),
            test_obj_gen_isim_data(imsi='234507095401016', mcc_mnc='23450',
                                   pattern_or_data='${IMSI}@ims.mnc${MNC}.mcc${MCC}.3gppnetwork.org',
                                   expected_op='234507095401016@ims.mnc050.mcc234.3gppnetwork.org'),
            test_obj_gen_isim_data(imsi='234507095401016', mcc_mnc='234507',
                                   pattern_or_data='${IMSI}@ims.mnc${MNC}.mcc${MCC}.3gppnetwork.org',
                                   expected_op='234507095401016@ims.mnc507.mcc234.3gppnetwork.org', ),
            test_obj_gen_isim_data(imsi='234507095401016', mcc_mnc='23450',
                                   pattern_or_data='sip:${IMSI}@ims.mnc${MNC}.mcc${MCC}.3gppnetwork.org',
                                   expected_op='sip:234507095401016@ims.mnc050.mcc234.3gppnetwork.org'),
            test_obj_gen_isim_data(imsi='234507095401016', mcc_mnc='234507',
                                   pattern_or_data='sip:${IMSI}@ims.mnc${MNC}.mcc${MCC}.3gppnetwork.org',
                                   expected_op='sip:234507095401016@ims.mnc507.mcc234.3gppnetwork.org'),
            test_obj_gen_isim_data(imsi='234507095401016', mcc_mnc='23450',
                                   pattern_or_data='pratik.jaiswal@montymobile.com',
                                   expected_op='pratik.jaiswal@montymobile.com'),
        ]
        for count, test_obj in enumerate(test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + str(count)):
                self.assertEqual(
                    PhUtil.gen_isim_data(imsi=test_obj.imsi, mcc_mnc=test_obj.mcc_mnc,
                                         pattern_or_data=test_obj.pattern_or_data),
                    test_obj.expected_op)

    def test_print_separator(self):
        """

        :return:
        """
        test_obj_pool = [
            test_obj_print_seperator(
                character='-', count=80,
                main_text='0',
                expected_op='--------------------------------------- 0 ---------------------------------------',
                tc_name='1 char (Odd)'),
            test_obj_print_seperator(
                character='-', count=80,
                main_text='01',
                expected_op='--------------------------------------- 01 ---------------------------------------',
                tc_name='2 char (Even)'),
            test_obj_print_seperator(
                character='-', count=80,
                main_text='01234567890123456',
                expected_op='------------------------------- 01234567890123456 -------------------------------',
                tc_name='Odd chars'),
            test_obj_print_seperator(
                character='-', count=80,
                main_text='0123456789012345',
                expected_op='-------------------------------- 0123456789012345 --------------------------------',
                tc_name='Even chars'),
            test_obj_print_seperator(
                character='-', count=80,
                main_text='01234567890123456789012345678901234567890123456789012345678901234567890123456789',
                expected_op='- 01234567890123456789012345678901234567890123456789012345678901234567890123456789 -',
                tc_name='chars same as count'),
            test_obj_print_seperator(
                character='-', count=80,
                main_text='012345678901234567890123456789012345678901234567890123456789012345678901234567',
                expected_op='- 012345678901234567890123456789012345678901234567890123456789012345678901234567 -',
                tc_name='chars = count -2 '),
            test_obj_print_seperator(
                character='-', count=80,
                main_text='0123456789012345678901234567890123456789012345678901234567890123456789012345',
                expected_op='- 0123456789012345678901234567890123456789012345678901234567890123456789012345 -',
                tc_name='chars = count -4'),
            test_obj_print_seperator(
                character='-', count=80,
                main_text=None,
                expected_op='--------------------------------------------------------------------------------',
                tc_name='main_text is None'),
            test_obj_print_seperator(
                character='-', count=80,
                main_text='-',
                expected_op='--------------------------------------- - ---------------------------------------'),
        ]
        for count, test_obj in enumerate(test_obj_pool, start=1):
            tc_name = test_obj.tc_name if test_obj.tc_name else str(count)
            with self.subTest(STR_TEST_OBJ + tc_name):
                self.assertEqual(
                    PhUtil.print_separator(character=test_obj.character, count=test_obj.count,
                                           main_text=test_obj.main_text, get_only=True)
                    , test_obj.expected_op)


if __name__ == '__main__':
    unittest.main()



import copy
import os
import sys
import time
import unittest
from collections import OrderedDict

import play_helpers
from play_helpers.ph_constants import PhConstants
from play_helpers.ph_constants_config import PhConfigConst
from play_helpers.ph_crypto import PhCrypto
from play_helpers.ph_defaults import PhDefaults
from play_helpers.ph_git import PhGit
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_misc import PhMisc
from play_helpers.ph_modules import PhModules, PhApps
from play_helpers.ph_time import PhTime
from play_helpers.ph_util import PhUtil
from play_helpers.test import test_util
from play_helpers.test.ph_constants_test import PhTestConst

_unit_test_mode = False


def test_version():
    PhUtil.print_heading()
    PhUtil.print_heading(str_heading=f'tool_version={PhDefaults.VERSION}')
    PhUtil.print_version('Test Tool', PhDefaults.VERSION)
    PhUtil.print_heading(str_heading='')
    PhUtil.print_version()
    PhUtil.print_heading(str_heading='tool_version=None')
    PhUtil.print_version('Test Tool', None)
    PhUtil.print_heading(str_heading='fetch_tool_version=True')
    PhUtil.print_version(PhModules.PLAY_HELPERS, PhDefaults.VERSION, fetch_tool_version=True)
    PhUtil.print_heading(str_heading='fetch_tool_version=False')
    PhUtil.print_version(PhModules.PLAY_HELPERS, PhDefaults.VERSION, fetch_tool_version=False)
    PhUtil.print_heading(str_heading='no_additional_info=True')
    PhUtil.print_version('Test Tool', PhDefaults.VERSION, no_additional_info=True)
    PhUtil.print_heading(str_heading='no_additional_info=False')
    PhUtil.print_version('Test Tool', PhDefaults.VERSION, no_additional_info=False)
    PhUtil.print_heading(str_heading='with_ip=True')
    PhUtil.print_version('Test Tool', PhDefaults.VERSION, with_ip=True)
    PhUtil.print_heading(str_heading='with_ip=False')
    PhUtil.print_version('Test Tool', PhDefaults.VERSION, with_ip=False)
    PhUtil.print_heading(str_heading='with_git_summary=True')
    PhUtil.print_version('Test Tool', PhDefaults.VERSION, with_git_summary=True)
    PhUtil.print_heading(str_heading='with_git_summary=False')
    PhUtil.print_version('Test Tool', PhDefaults.VERSION, with_git_summary=False)
    PhUtil.print_heading(str_heading='with_git_detailed_info=True')
    PhUtil.print_version('Test Tool', PhDefaults.VERSION, with_git_detailed_info=True)
    PhUtil.print_heading(str_heading='with_git_detailed_info=False')
    PhUtil.print_version('Test Tool', PhDefaults.VERSION, with_git_detailed_info=False)
    PhUtil.print_heading(str_heading='version_parameters_dicts')
    version_parameters_pool = [
        {'tool_name': PhConfigConst.TOOL_NAME, 'tool_version': PhConfigConst.TOOL_VERSION},
        {'tool_name': 'packaging', 'tool_version': None, 'fetch_tool_version': True},
        {'tool_name': 'pandas', 'fetch_tool_version': True},
        {'tool_name': 'psutil', 'fetch_tool_version': True},
        {'tool_name': 'tzlocal', 'fetch_tool_version': True},
        {'tool_name': 'requests', 'fetch_tool_version': True},
        {'tool_name': 'pycryptodome', 'fetch_tool_version': True},
        {'tool_name': 'ruamel.yaml', 'fetch_tool_version': True},
    ]
    PhUtil.print_version(parameters_pool=version_parameters_pool)
    PhUtil.print_heading(str_heading='parameters_pool=version_parameters_pool, get_only=True, dic_format=True')
    PhUtil.print_iter(
        the_iter=PhUtil.print_version(parameters_pool=version_parameters_pool, get_only=True, dic_format=True),
        header='Pool')


def test_misc():
    PhUtil.print_heading()
    print(PhUtil.len_even('123456'))
    print(PhUtil.len_odd('123456'))


def test_list():
    PhUtil.print_heading()
    PhUtil.print_heading(str_heading='cast_to_list')
    print(PhUtil.to_list(5))
    print(PhUtil.to_list('5'))
    print(PhUtil.to_list('6   ', trim_data=False))
    print(PhUtil.to_list('6   ', trim_data=True))
    print(PhUtil.to_list(['5']))
    print(PhUtil.to_list(None))
    print(PhUtil.to_list([]))
    PhUtil.print_heading(str_heading='extend_list')
    print(PhUtil.extend_list([6], 5))
    print(PhUtil.extend_list(['Pj'], 9))
    print(PhUtil.extend_list(['Pj'], 1))
    print(PhUtil.extend_list(['Pj'], 0))
    print(PhUtil.extend_list([], 5))
    print(PhUtil.extend_list(None, 5))
    print(PhUtil.extend_list(None, 5, filler='Pj'))
    print(PhUtil.extend_list(['Pj', 'Jp'], 5, filler='P'))
    PhUtil.print_heading(str_heading='extend_list, unique')
    print(PhUtil.extend_list([6], 5, unique_entries=True))
    print(PhUtil.extend_list(['Pj'], 9, unique_entries=True))
    print(PhUtil.extend_list(['Pj'], 1, unique_entries=True))
    print(PhUtil.extend_list(['Pj'], 0, unique_entries=True))
    print(PhUtil.extend_list([], 5, unique_entries=True))
    print(PhUtil.extend_list(None, 5, unique_entries=True))
    print(PhUtil.extend_list(None, 5, filler='Pj', unique_entries=True))
    print(PhUtil.extend_list(['Pj', 'Jp'], 5, filler='P', unique_entries=True))
    PhUtil.print_heading(str_heading='extend_list, unique, trim data')
    print(PhUtil.extend_list([' Pj', 'Jp '], 5, filler='P', unique_entries=True, trim_data=True))
    print(PhUtil.extend_list([' Pj', 'Jp '], 5, filler='P', unique_entries=True, trim_data=False))
    print(PhUtil.extend_list([' Pj', 'Jp '], 5, filler=' P', unique_entries=True, trim_data=True))
    print(PhUtil.extend_list([' Pj', 'Jp '], 5, filler=' P', unique_entries=True, trim_data=False))
    PhUtil.print_heading(str_heading='combine_list_items')
    print(PhUtil.combine_list_items(['Pj', 'Lp', ' nn'], trim_data=True))
    print(PhUtil.combine_list_items(['Pj', 'Lp', ' nn'], trim_data=False))
    print(PhUtil.combine_list_items('Pj'))
    PhUtil.print_heading(str_heading='combine_list_items ; clean_data=False')
    print(PhUtil.combine_list_items([';Pj ', 'Lp', 'nn'], clean_data=False))
    print(PhUtil.combine_list_items(['Pj; ', 'Lp', 'nn'], clean_data=False))
    print(PhUtil.combine_list_items(['Pj; ', 'Lp;', 'nn'], clean_data=False))
    print(PhUtil.combine_list_items(['Pj; ', 'Lp;', 'nn;'], clean_data=False))
    print(PhUtil.combine_list_items(['Pj; ', '   ; Lp;', 'nn'], clean_data=False))
    print(PhUtil.combine_list_items(['P;j; ', 'Lp;', 'nn'], clean_data=False))
    print(PhUtil.combine_list_items(['# BulkMode; DirectoryInput; YmlInput; .YmlFiles;', '(3 Elements)'],
                                    clean_data=False))
    print(PhUtil.combine_list_items(['Pj;'], clean_data=False))
    print(PhUtil.combine_list_items(['Remarks semi colon 1;', 'Remarks semi colon 2; '], clean_data=False))
    print(PhUtil.combine_list_items(['Remarks semi colon 1;', 'Remarks semi colon 2; ; '], clean_data=False))
    PhUtil.print_heading(str_heading='combine_list_items ; clean_data=True')
    print(PhUtil.combine_list_items([';Pj ', 'Lp', 'nn'], clean_data=True))
    print(PhUtil.combine_list_items(['Pj; ', 'Lp', 'nn'], clean_data=True))
    print(PhUtil.combine_list_items(['Pj; ', 'Lp;', 'nn'], clean_data=True))
    print(PhUtil.combine_list_items(['Pj; ', 'Lp;', 'nn;'], clean_data=True))
    print(PhUtil.combine_list_items(['Pj; ', '   ; Lp;', 'nn'], clean_data=True))
    print(PhUtil.combine_list_items(['P;j; ', 'Lp;', 'nn'], clean_data=True))
    print(PhUtil.combine_list_items(['# BulkMode; DirectoryInput; YmlInput; .YmlFiles;', '(3 Elements)'],
                                    clean_data=True))
    print(PhUtil.combine_list_items(['Pj;'], clean_data=True))
    print(PhUtil.combine_list_items(['Remarks semi colon 1;', 'Remarks semi colon 2; '], clean_data=True))
    print(PhUtil.combine_list_items(['Remarks semi colon 1;', 'Remarks semi colon 2; ; '], clean_data=True))
    print(PhUtil.combine_list_items([5, 6, 7], clean_data=True))


def test_heading():
    PhUtil.print_heading()
    PhUtil.print_heading(
        str_heading=r'..\..\Data\UserData\GSMA\SGP_22\v3_0_0\UpdateMetadataRequest\DerInput_Asn1Output_DirectInput_Asn1Element_UpdateMetadataRequest_ExportKeyword_OutputFile_export.yml')
    PhUtil.print_heading(
        str_heading='This is a long dataaaa for the testing of Trimming of remarks of individual item of the pool in asn play.; Item 100')
    PhUtil.print_heading(
        str_heading='This is a long dataaaaa for the testing of Trimming of remarks of individual item of the pool in asn play.; Item 100')
    PhUtil.print_heading(
        str_heading='This is a long dataaaaaa for the testing of Trimming of remarks of individual item of the pool in asn play.; Item 100')
    PhUtil.print_heading(
        str_heading='This is a long dataaaaaaaaa for the testing of Trimming of remarks of individual item of the pool in asn play.; Item 100')
    PhUtil.print_heading(
        str_heading='This is a long dataaaaaaaaaaaaaaaaaaaaa for the testing of Trimming of remarks of individual item of the pool in asn play.; Item 100')


def test_python_friendly_name():
    PhUtil.print_heading()
    print(PhUtil.get_python_friendly_name(
        'Error ruamel.yaml.representer.RepresenterError: cannot represent an object: <ProfileInfoListResponse (CHOICE)>...'))
    print(PhUtil.get_python_friendly_name(
        'Error ruamel.yaml.representer.RepresenterError: cannot represent an object: <ProfileInfoListResponse (CHOICE)>...; item 1'))
    print(PhUtil.get_python_friendly_name(
        'Error ruamel.yaml.representer.RepresenterError: cannot represent an object: <ProfileInfoListResponse (CHOICE)>'))
    print(PhUtil.get_python_friendly_name('#userInput'))
    print(PhUtil.get_python_friendly_name('userInput#'))
    print(PhUtil.get_python_friendly_name('#userInput#'))


def test_remarks_append_post():
    PhUtil.print_heading()
    print(PhUtil.append_remarks('StoreMetadataRequest',
                                '..\\..\\Data\\SampleData\\GSMA\\SGP_22\\v3_0_0\\StoreMetadataRequest\\StoreMetadataRequest_wo_icon.hex'))
    print(PhUtil.append_remarks('StoreMetadataRequest',
                                '..\\..\\Data\\SampleData\\GSMA\\SGP_22\\v3_0_0\\StoreMetadataRequest\\StoreMetadataRequest_Mandatory.hex'))
    print(PhUtil.append_remarks('StoreMetadataRequest',
                                'BF2581885A0A989209012143658709F591095350204E616D652031921A4F7065726174696F6E616C2050726F66696C65204E616D652031930101B621301F800204F0811974657374736D6470706C7573312E6578616D706C652E636F6DB705800392F91899020640BF220F300D8003883710A1060404C1020304BF230F300D8003883711A106040402020202'))
    print(PhUtil.append_remarks('',
                                'StoreMetadataRequest; BF2581885A0A989209012143658709F591095350204E616D652031921A4F7065726174696F6E616C2050726F66696C6...'))
    print(PhUtil.append_remarks(
        'StoreMetadataRequest; BF2581885A0A989209012143658709F591095350204E616D652031921A4F7065726174696F6E616C2050726F66696C6...',
        ''))
    print(PhUtil.append_remarks('',
                                'BF2581885A0A989209012143658709F591095350204E616D652031921A4F7065726174696F6E616C2050726F66696C65204E616D652031930101B621301F800204F0811974657374736D6470706C7573312E6578616D706C652E636F6DB705800392F91899020640BF220F300D8003883710A1060404C1020304BF230F300D8003883711A106040402020202'))
    print(PhUtil.append_remarks(
        'BF2581885A0A989209012143658709F591095350204E616D652031921A4F7065726174696F6E616C2050726F66696C65204E616D652031930101B621301F800204F0811974657374736D6470706C7573312E6578616D706C652E636F6DB705800392F91899020640BF220F300D8003883710A1060404C1020304BF230F300D8003883711A106040402020202'))
    print(PhUtil.append_remarks('', 'item 100'))
    print(PhUtil.append_remarks('T', 'item 100'))
    print(PhUtil.append_remarks(
        'This is a long data for the testing of Trimming of remarks of individual item of the pool in asn play.',
        'item 100'))
    for i in range(1, 120):
        print(PhUtil.append_remarks(
            'a' * i + ' This is a long data for the testing of Trimming of remarks of individual item of the pool in asn play.',
            'item 100'))
        i = i + 1
    print(PhUtil.append_remarks(
        'This is a long dataaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa for the testing of Trimming of remarks of individual item of the pool in asn play.',
        'item 100'))


def test_remarks_append_pre():
    PhUtil.print_heading()
    print(PhUtil.append_remarks('item 100', '', append_mode_post=False))
    print(PhUtil.append_remarks('item 100', 'T', append_mode_post=False))
    print(PhUtil.append_remarks(
        'item 100',
        'This is a long data for the testing of Trimming of remarks of individual item of the pool in asn play.',
        append_mode_post=False
    ))
    for i in range(1, 120):
        print(PhUtil.append_remarks(
            'item 100',
            'a' * i + ' This is a long data for the testing of Trimming of remarks of individual item of the pool in asn play.',
            append_mode_post=False
        ))
        i = i + 1
    print(PhUtil.append_remarks(
        'item 100',
        'This is a long dataaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa for the testing of Trimming of remarks of individual item of the pool in asn play.',
        append_mode_post=False))
    for i in range(1, 120):
        print(PhUtil.append_remarks(
            'item 100' + '0' * i,
            ' This is a long data for the testing of Trimming of remarks of individual item of the pool in asn play.',
            append_mode_post=False
        ))
        i = i + 1


def test_temp():
    PhUtil.print_heading()
    # test_open_bugs()


def test_open_bugs():
    PhUtil.print_heading()
    #
    # Package not found
    #
    # Expected Output:
    #   ruamel.yaml: v0.18.6
    # Actual Output:
    #   Known Exception Occurred; Summary: package not found error; Details: ruamel.yaml
    #   ruamel.yaml version is -1.-1.-1
    PhUtil.print_version('ruamel.yaml', fetch_tool_version=True)


def test_hash():
    PhUtil.print_heading()
    PhUtil.print_heading(str_heading='hash_algos_list')
    print(PhCrypto.hash_algos_list())
    msg = 'First'
    PhUtil.print_heading(str_heading='Str hash')
    print(f'Input String: {msg}; hash_str_sha256: {PhCrypto.hash_str_sha256(msg=msg)}')
    print(f'Input String: {msg}; hash_str: {PhCrypto.hash_str(msg=msg)}')
    print(f'Input String: {msg}; hash_str(hash_algo=sha512) {PhCrypto.hash_str(msg=msg, hash_algo="sha512")}')


def test_get_git_info():
    PhUtil.print_heading()
    PhUtil.print_heading('get_git_info_detailed; GIT_SUMMARY')
    print(f'GIT_SUMMARY: {PhGit.get_git_info_detailed(key=PhGit.KEY_GIT_SUMMARY)}')
    PhUtil.print_heading('get_git_info_detailed')
    print(PhGit.get_git_info_detailed())
    PhUtil.print_heading('get_git_info_detailed iter')
    PhUtil.print_iter(PhGit.get_git_info_detailed())


def test_get_time_stamp_file_name():
    PhUtil.print_heading()
    outputs = []
    for _ in range(15):
        data = PhUtil.get_time_stamp(files_format=True)
        print(f'{data[:13]} {data[13:15]} {data[15:]}')
    for _ in range(500):
        outputs.append(PhUtil.get_time_stamp(files_format=True))
    dup_list = [x for x in outputs if outputs.count(x) > 1]
    PhUtil.print_iter(dup_list, header='Duplicate Items')


def test_to_file():
    PhUtil.print_heading()
    PhUtil.to_file(output_lines='abc', file_name='abc.txt')
    PhUtil.to_file(output_lines='abc', file_name=os.sep.join([PhUtil.path_default_data_folder, 'abc.txt']))
    PhUtil.to_file(output_lines='abc', file_name=os.sep.join([PhUtil.path_default_data_folder, 'abc.list']))
    PhUtil.to_file(output_lines='abc', file_name=os.sep.join([PhUtil.path_default_data_folder, 'abc.ini']))
    PhUtil.to_file(output_lines=['abc', 'def', 'ghi'],
                   file_name=os.sep.join([PhUtil.path_default_data_folder, 'abc_list.txt']))
    PhUtil.to_file(output_lines='abc', back_up_file=True)
    PhUtil.to_file(output_lines='abc', file_name=os.sep.join([PhUtil.path_default_bkp_folder, 'a_b_c.txt']),
                   back_up_file=True)
    PhUtil.to_file(output_lines='abc', file_name=PhUtil.path_default_bkp_folder, back_up_file=True,
                   file_path_is_dir=True)


def test_print_iter():
    PhUtil.print_heading()
    dic1 = {
        'raw_data': '',
        'input_format': 'url',
        'remarks_list': '',
        'sample_data': 'AmenityPj',
        'sample_option': 'Load Only',
        'sample_process': '',
    }
    dic2 = {
        'app_title': '',
        'app_description': 'url',
        'app_version': '',
        'app_github_url': 'AmenityPj',
        'app_github_pages_url': 'Load Only',
    }
    dic3 = {
        '1': dic1,
        '2': dic2,
    }
    dic4 = {}
    dic5 = {'app_title': ''}
    list1 = ['raw_data', 'input_format', 'remarks_list', 'sample_data', 'sample_option', 'sample_process']
    list2 = ['app_title', 'app_description', 'app_version', 'app_github_url', 'app_github_pages_url']
    list3 = [list1, list2]
    list4 = []
    list5 = ['app_title']
    ordered_dic1 = OrderedDict()
    ordered_dic1.update({'dic1': dic1})
    ordered_dic1.update({'dic2': dic2})
    ordered_dic1.update({'dic3': dic3})
    sep = PhConstants.SEPERATOR_MULTI_OBJ
    list_of_items = [
        [dic1, 'dic1'],
        [dic2, 'dic2'],
        [dic3, 'dic3'],
        [dic4, 'dic4'],
        [dic5, 'dic5'],
        [list1, 'list1'],
        [list2, 'list2'],
        [list3, 'list3'],
        [list4, 'list4'],
        [list5, 'list5'],
        [ordered_dic1, 'orderedDic1'],
    ]

    PhUtil.print_heading(str_heading='Simple Iters')
    PhUtil.print_iter(dic1, header='dic1')
    PhUtil.print_iter(list1, header='list1')
    PhUtil.print_iter(list1, header='list1; list_as_str=True', list_as_str=True)

    PhUtil.print_heading(str_heading='iters printing with heading')
    for item in list_of_items:
        actual_iter = item[0]
        header = item[1]
        PhUtil.print_iter(actual_iter, header=header)

    PhUtil.print_heading(str_heading='iters printing')
    for item in list_of_items:
        PhUtil.print_iter(item[0])

    PhUtil.print_heading(str_heading='iters printing; depth_level=1')
    for item in list_of_items:
        PhUtil.print_iter(item[0], depth_level=1)

    PhUtil.print_heading(str_heading='iters printing, with sep')
    for item in list_of_items:
        PhUtil.print_iter(item[0], sep=sep)

    data = sys.modules
    PhUtil.print_heading(str_heading='sys.modules; depth_level=0')
    PhUtil.print_iter(data, depth_level=0)
    PhUtil.print_heading(str_heading='sys.modules; check_for_dict_attribute=False')
    PhUtil.print_iter(data, check_for_dict_attribute=False)
    # TODO: https://pratikj.atlassian.net/browse/SML-398
    # PhUtil.print_heading(str_heading='sys.modules')
    # PhUtil.print_iter(data)

    PhUtil.print_heading(str_heading='class')
    PhUtil.print_iter(PhConstants)

    PhUtil.print_heading(str_heading='enum')
    PhUtil.print_iter(PhConstants.Position)

    PhUtil.print_heading(str_heading='enum item')
    PhUtil.print_iter(PhConstants.Position.TOP)

    PhUtil.print_heading(str_heading='dict with enum')
    dic_w_enum = {
        'app_title': '',
        'app_description': 'url',
        'app_version': '',
        'app_github_url': 'AmenityPj',
        'app_github_pages_url': 'Load Only',
        'position': PhConstants.Position.TOP,
        'items': PhConstants.Position,
    }
    PhUtil.print_iter(dic_w_enum)


def test_obj_list():
    PhUtil.print_heading()
    PhUtil.print_heading(str_heading='cls_to_explore=PhUtil')
    PhUtil.get_obj_list(cls_to_explore=PhUtil, print_also=True)
    PhUtil.print_heading(str_heading="cls_to_explore=PhUtil, obj_name_filter='print'")
    PhUtil.get_obj_list(cls_to_explore=PhUtil, obj_name_filter='print', print_also=True)
    PhUtil.print_heading(str_heading='cls_to_explore=play_helpers')
    PhUtil.get_obj_list(cls_to_explore=play_helpers, print_also=True)
    PhUtil.print_heading(str_heading="cls_to_explore=play_helpers, obj_name_filter='ph'")
    PhUtil.get_obj_list(cls_to_explore=play_helpers, obj_name_filter='ph', print_also=True)


def test_get_classes_list():
    PhUtil.print_heading()
    PhUtil.print_heading(str_heading='module_to_explore=play_helpers.ph_util')
    PhUtil.get_classes_list(module_to_explore=play_helpers.ph_util, print_also=True)
    PhUtil.print_heading(str_heading='module_to_explore=play_helpers.ph_util, obj_name_needed=False')
    PhUtil.get_classes_list(module_to_explore=play_helpers.ph_util, obj_name_needed=False, print_also=True)
    PhUtil.print_heading(str_heading='module_to_explore=test_util')
    PhUtil.get_classes_list(module_to_explore=test_util, print_also=True)
    PhUtil.print_heading(str_heading='module_to_explore=test_util, parent_class=unittest.TestCase')
    PhUtil.get_classes_list(module_to_explore=test_util, parent_class=unittest.TestCase, print_also=True)


def test_print_modules():
    PhUtil.print_heading()
    PhUtil.print_heading(str_heading='No filter String')
    PhUtil.print_modules()
    PhUtil.print_heading(str_heading="filter_string='play_helpers'")
    PhUtil.print_modules(filter_string='play_helpers')


def test_print_all_installed_distributions():
    PhUtil.print_heading()
    PhUtil.print_heading(str_heading='dic_format=True')
    PhUtil.print_all_installed_distributions(dic_format=True)
    PhUtil.print_heading(str_heading='dic_format=False')
    PhUtil.print_all_installed_distributions(dic_format=False)


def test_generalise_list():
    PhUtil.print_heading()
    list1 = [1, 2, 3, 4, 5]
    PhUtil.print_iter(list1, header='list1')
    generalise_list = PhUtil.generalise_list(list1)
    PhUtil.print_iter(generalise_list, header='generalise_list')
    generalise_list_rev = PhUtil.generalise_list_reverse(generalise_list)
    PhUtil.print_iter(generalise_list_rev, header='generalise_list_rev')
    list1_rev = PhUtil.generalise_list_reverse(list1)
    PhUtil.print_iter(list1_rev, header='list1_rev')
    generalise_list_append_others = PhUtil.generalise_list(list1, append_others=True)
    PhUtil.print_iter(generalise_list_append_others, header='generalise_list_append_others')
    generalise_list_append_others_rev = PhUtil.generalise_list_reverse(generalise_list_append_others)
    PhUtil.print_iter(generalise_list_append_others_rev, header='generalise_list_append_others_rev')


def test_zipfile():
    PhUtil.print_heading()
    source_files_dir = PhUtil.path_default_data_folder
    target_dir = PhUtil.path_default_res_folder
    PhUtil.print_heading(str_heading='keep_source_dir_in_zip=True')
    PhUtil.zip_and_clean_dir(source_files_dir=source_files_dir, target_dir=target_dir, keep_source_dir_in_zip=True,
                             target_file_name_wo_extn='test_zipfile_keep_source_dir_in_zip_true')
    PhUtil.print_heading(str_heading='keep_source_dir_in_zip=False')
    PhUtil.zip_and_clean_dir(source_files_dir=source_files_dir, target_dir=target_dir, keep_source_dir_in_zip=False,
                             target_file_name_wo_extn='test_zipfile_keep_source_dir_in_zip_false')
    PhUtil.print_heading(str_heading='source_files_dir=target_dir')
    PhUtil.zip_and_clean_dir(source_files_dir=source_files_dir, target_dir=source_files_dir,
                             target_file_name_wo_extn='test_zipfile_keep_source_path_is_dest_path')
    PhUtil.print_heading(str_heading='source_files_and_target_dir')
    PhUtil.zip_and_clean_dir(source_files_dir=source_files_dir, target_dir=target_dir)
    PhUtil.print_heading(str_heading='only_source_files')
    PhUtil.zip_and_clean_dir(source_files_dir=source_files_dir)
    PhUtil.print_heading(str_heading='source_dir_custom_extn_files')
    file_list = ['*.txt']
    PhUtil.zip_and_clean_dir(source_files_dir=source_files_dir, include_files=file_list,
                             target_file_name_wo_extn='test_zipfile_source_dir_custom_extn_files')


def test_chars_to_utf8():
    PhUtil.print_heading()
    target_file_name = 'test_chars_to_utf8.asn'
    lines = []
    for encoding in PhConstants.CHAR_ENCODING_POOL:
        PhUtil.print_heading(f'encoding: {encoding}')
        with open(os.sep.join([PhUtil.path_default_res_folder, target_file_name]),
                  encoding=encoding, errors=PhDefaults.CHAR_ENCODING_ERRORS) as f:
            lines = f.readlines()
        for input_data in lines:
            output_data = PhUtil.all_chars_to_utf8(input_data)
            data = PhConstants.SEPERATOR_MULTI_LINE_TABBED.join(
                [f'input_data: {input_data}', f'output_data: {output_data}'])
            print(data)


def test_traverse_it():
    PhUtil.print_heading()
    """
    Test Data
    """
    #
    target_file1_name = PhTestConst.SRC_CODE_PH_BITS_FILE_NAME
    target_file2_name = PhTestConst.SRC_CODE_PH_CONSTANTS_FILE_NAME
    target_file3_name = PhTestConst.SRC_CODE_PH_DATA_MASTER_FILE_NAME
    target_file1_name_star_w_ext = 'ph_bit*.py'
    target_file2_name_star_w_ext = 'ph_constant*.py'
    target_file3_name_star_w_ext = 'ph_data_*.py'
    target_file1_name_star_wo_ext = 'ph_bit*'
    target_file2_name_star_wo_ext = 'ph_constant*'
    target_file3_name_star_wo_ext = 'ph_data_*'
    target_file1_path = PhTestConst.SRC_CODE_PH_BITS_FILE_PATH
    target_file2_path = PhTestConst.SRC_CODE_PH_CONSTANTS_FILE_PATH
    target_file3_path = PhTestConst.SRC_CODE_PH_DATA_MASTER_FILE_PATH
    target_dir_name = PhTestConst.SRC_CODE_TEST_DIR_NAME
    target_dir_path = PhTestConst.SRC_CODE_TEST_DIR_PATH
    #
    src_dir_rel = os.sep.join([PhUtil.path_current_folder, os.pardir])
    src_dir = PhUtil.get_absolute_path(src_dir_rel)
    src_file_nok = target_file1_path
    #
    inc_files_pattern1_ok = '*.py'
    inc_files_pattern2_ok = '*__.py'
    inc_files_pattern_nok = ['*_*.py']
    inc_files_pattern_dual_ok = ['*.py', '*.txt']
    #
    inc_files_name_nok = target_file1_name
    inc_files_name_pattern_ok = '*' + target_file1_name
    inc_files_names_pattern_multi_ok = [
        '*' + target_file1_name,
        '*' + target_file2_name,
        '*' + target_file3_name
    ]
    inc_files_name_pattern_star_w_ext_ok = '*' + target_file1_name_star_w_ext
    inc_files_name_pattern_star_wo_ext_ok = '*' + target_file1_name_star_wo_ext
    inc_files_name_pattern_star_w_ext_multi_ok = [
        '*' + target_file1_name_star_w_ext,
        '*' + target_file2_name_star_w_ext,
        '*' + target_file3_name_star_w_ext
    ]
    inc_files_name_pattern_star_wo_ext_multi_ok = [
        '*' + target_file1_name_star_wo_ext,
        '*' + target_file2_name_star_wo_ext,
        '*' + target_file3_name_star_wo_ext
    ]
    #
    inc_files_path_ok = target_file1_path
    inc_files_path_multi_ok = [target_file1_path, target_file2_path, target_file3_path]
    inc_files_path_pattern_ok = '*' + target_file1_path
    inc_files_path_pattern_multi_ok = ['*' + target_file1_path, '*' + target_file2_path, '*' + target_file3_path]
    #
    exc_file_name_nok = target_file1_name
    exc_dir_name_nok = target_dir_name
    exc_file_path_ok = target_file1_path
    exc_dir_path_ok = target_dir_path
    #
    exc_files_pattern_ok = '*.pyc'
    """
    Test Cases
    """
    #
    PhUtil.print_heading(str_heading='No Args; Working')
    PhUtil.traverse_it(print_also=True)
    #
    PhUtil.print_heading(str_heading='include_files=inc_files_pattern_dual_ok')
    PhUtil.traverse_it(print_also=True, include_files=inc_files_pattern_dual_ok)
    #
    PhUtil.print_heading(str_heading='src_dir_rel')
    PhUtil.traverse_it(print_also=True, top=src_dir_rel)
    #
    PhUtil.print_heading(str_heading='src_dir_rel, include_files=inc_files_pattern1_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir_rel, include_files=inc_files_pattern1_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_pattern1_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_pattern1_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_pattern2_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_pattern2_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_pattern_nok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_pattern_nok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_name_nok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_name_nok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_name_pattern_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_name_pattern_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_names_pattern_multi_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_names_pattern_multi_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_name_pattern_star_w_ext_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_name_pattern_star_w_ext_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_name_pattern_star_w_ext_multi_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_name_pattern_star_w_ext_multi_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_name_pattern_star_wo_ext')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_name_pattern_star_wo_ext_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_name_pattern_star_wo_ext_multi_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_name_pattern_star_wo_ext_multi_ok)
    #
    PhUtil.print_heading(
        str_heading='src_dir, include_files=inc_files_name_pattern_star_wo_ext_multi_ok, excludes=exc_files_pattern_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_name_pattern_star_wo_ext_multi_ok,
                       excludes=exc_files_pattern_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_path_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_path_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_path_multi_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_path_multi_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_path_pattern_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_path_pattern_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, include_files=inc_files_path_pattern_multi_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, include_files=inc_files_path_pattern_multi_ok)
    #
    PhUtil.print_heading(str_heading='src_file_nok, include_files=inc_files_pattern1_ok')
    PhUtil.traverse_it(print_also=True, top=src_file_nok, include_files=inc_files_pattern1_ok)
    #
    PhUtil.print_heading(str_heading='src_file_nok')
    PhUtil.traverse_it(print_also=True, top=src_file_nok)
    #
    PhUtil.print_heading(str_heading='src_dir, excludes=exc_file_name_nok')
    PhUtil.traverse_it(print_also=True, top=src_dir, excludes=exc_file_name_nok)
    #
    PhUtil.print_heading(str_heading='src_dir, excludes=exc_dir_name_nok')
    PhUtil.traverse_it(print_also=True, top=src_dir, excludes=exc_dir_name_nok)
    #
    PhUtil.print_heading(str_heading='src_dir, excludes=exc_file_path_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, excludes=exc_file_path_ok)
    #
    PhUtil.print_heading(str_heading='src_dir, excludes=exc_dir_path_ok')
    PhUtil.traverse_it(print_also=True, top=src_dir, excludes=exc_dir_path_ok)


def test_eid():
    PhUtil.print_heading()
    eid_pool = [
        '89049032007008882600117893640261',
        '89049032123451234512345678901235',
    ]
    for eid in eid_pool:
        print(f'EID {eid} is Valid ? {PhCrypto.validate_eid(eid)}')


def test_luhn():
    PhUtil.print_heading()
    PhUtil.print_heading(str_heading='debit_card_pool')
    debit_card_pool = [
        '4722 5410 5101 6295',
    ]
    for debit_card in debit_card_pool:
        print(f'Debit Card {debit_card} is Valid ? {PhCrypto.validate_luhn_digit(debit_card)}')
    PhUtil.print_heading(str_heading='credit_card_pool')
    credit_card_pool = [
        '4386 2800 2898 6424',
    ]
    for cebit_card in credit_card_pool:
        print(f'Cebit Card {cebit_card} is Valid ? {PhCrypto.validate_luhn_digit(cebit_card)}')
    PhUtil.print_heading(str_heading='iccid_pool')
    iccid_pool = [
        '894461721200000114',
        '8944617212000001146',
        '89423 10220 00089 671 6',
    ]
    for iccid in iccid_pool:
        print(f'ICCID {iccid} is Valid ? {PhCrypto.validate_luhn_digit(iccid)}')
    PhUtil.print_heading(str_heading='imei_pool')
    imei_pool = [
        '501944690893758',
        '910107118819392',
        '502652252218376',
    ]
    for imei in imei_pool:
        print(f'imei {imei} is Valid ? {PhCrypto.validate_luhn_digit(imei)}')


def test_handle_dirs():
    PhUtil.print_heading()
    PhUtil.print_heading('quite_mode=True; Default')
    PhUtil.make_dirs(dir_path='sample1')
    PhUtil.remove_dirs(dir_path='sample1')
    PhUtil.print_heading('quite_mode=False')
    PhUtil.make_dirs(dir_path='sample2', quite_mode=False)
    PhUtil.remove_dirs(dir_path='sample2', quite_mode=False)


def test_normalise_list():
    PhUtil.print_heading()
    data = [1, 2, 3, 4, 5, [6, 7, 8], 9]
    for x in range(5):
        res = PhUtil.normalise_list(data)
        PhUtil.print(res)


def test_time_delay(ph_time):
    ph_time.print()
    time.sleep(5)
    ph_time.print()


def test_doc_string():
    PhUtil.print_heading()
    func = PhCrypto.hash_str
    PhUtil.print_heading('help')
    help(func)
    PhUtil.print_heading('__doc__')
    print(func.__doc__)


def test_get_help_for_param():
    PhUtil.print_heading()
    PhUtil.print(PhUtil.get_help_for_param('Sample Param'))
    PhUtil.print(PhUtil.get_help_for_param('Sample Param with Valid Default Value', '5'))
    PhUtil.print(
        PhUtil.get_help_for_param('Sample Param with None Default Value; include_none=False', None, include_none=False))
    PhUtil.print(
        PhUtil.get_help_for_param('Sample Param with None Default Value; include_none=True', None, include_none=True))


def test_parse_config():
    PhUtil.print_heading()
    test_data_set = [
        {
            'remarks': 'Simple Data Types',
            'remarks_str01': 'Simple Data Types',
            'remarks_str02': ' Simple Data Types ',
            'remarks_str03': "  Simple Data Types  ",
            'remarks_str04': '"    Simple Data Types  "  ',
            'remarks_str05': '   """Simple Data Types  """',
            'remarks_str06': " '     Simple Data Types ' ",
            'remarks_str07': '86020102',
            'remarks_str08': 'False',
            'remarks_str09': 'True',
            'remarks_str0A': '86020102.0',
            'remarks_str0B': 'None',
            'item_s01': 'False',
            'item_s02': False,
            'item_s03': 'false',
            'item_s04': 'Fals e',
            'item_s05': 'FaLse',
            'item_s06': 'False ',
            'item_s07': ' False ',
            'item_s08': '"False"',
            'item_s11': 'True',
            'item_s12': True,
            'item_s13': 'true',
            'item_s14': 'Tr ue',
            'item_s15': 'TruE',
            'item_s16': 'True ',
            'item_s17': '   True ',
            'item_s18': '"True"',
            'item_s21': 'yes',
            'item_s22': 'Yes',
            'item_s23': 'no',
            'item_s24': 'No',
            'item_s25': ' ',
            'item_s26': "",
            'item_s27': """""",
            'item_s31': '86020102',
            'item_s32': '086020102',
            'item_s33': '8 6020102',
            'item_s34': '860201020 ',
            'item_s35': '   860201020 ',
            'item_s36': '86020102.0',
            'item_s37': '86020102.0 ',
            'item_s38': """
                            86020102.0
                        """,
            'item_s39': '86020102.064654767',
            'item_s3A': "'86020102'",
            'item_s3B': '"86020102"',
            'item_s41': 'none',
            'item_s42': None,
            'item_s43': 'None',
            'item_s44': 'NONE',
        },
        {
            'remarks': 'Complex Data Types',
            'item_c1': "[{'brown': 'black'}, {'dog': 'cattttttt'}]",
            'item_c2': [{'brown': 'black'}, {'dog': 'cattttttt'}],
            'item_c3': '[]',
            'item_c4': [],
            'item_c5': "{1: {'brown': 'black'},2: {'dog': 'cattttttt'}}",
            'item_c6': {1: {'brown': 'black'}, 2: {'dog': 'cattttttt'}},
            'item_c7': '{}',
            'item_c8': {},
        },
    ]
    remarks_exclude = [int, float, bool]
    for test_data in test_data_set:
        PhUtil.print_heading(test_data.get(PhKeys.REMARKS))
        PhUtil.print_iter(the_iter=test_data, verbose=True, header='Before')
        new_config = PhUtil.dict_to_data(copy.copy(test_data))
        PhUtil.print_iter(the_iter=new_config, verbose=True, header='After')
        # Prepare Exclude
        exclude_data = {}
        for key in test_data:
            if key.startswith('remarks_str'):
                exclude_data.update({key: remarks_exclude})
        new_config = PhUtil.dict_to_data(copy.copy(test_data), data_types_exclude=exclude_data)
        PhUtil.print_iter(the_iter=new_config, verbose=True, header='After Exclude')


def test_expired_attr():
    PhUtil.print_heading()
    # TODO: This does not work as expected, check implementation in Numpy
    # makedirs()


def test_trim_white_spaces_in_str():
    PhUtil.print_heading()
    input_data_set = [
        '       ',
        ' rgdrg  rtert',
        ' rgdrg                          rtert',
        """   rgdrg                      rte
            rt""",
        '\85',
        '\85 ',
        '\x85',
        ' \85 ',
    ]
    for input_data in input_data_set:
        output_data_trim_white_spaces_in_str = PhUtil.trim_white_spaces_in_str(input_data)
        output_data_trim_and_kill_all_white_spaces = PhUtil.trim_and_kill_all_white_spaces(input_data)
        output_data_trim_and_kill_extra_white_spaces = PhUtil.trim_and_kill_extra_white_spaces(input_data)
        PhUtil.print_input_output(
            input_data=input_data,
            output_data=
            [
                output_data_trim_white_spaces_in_str,
                output_data_trim_and_kill_all_white_spaces,
                output_data_trim_and_kill_extra_white_spaces
            ],
            output_remarks=
            ['trim_white_spaces_in_str        ',
             'trim_and_kill_all_white_spaces  ',
             'trim_and_kill_extra_white_spaces',
             ],
            input_remark
            ='input Data                      ',
            verbose=True,
        )


def test_last_modification_time():
    PhUtil.print_heading()
    path_current_script = __file__
    name_current_script = os.path.basename(__file__)
    input_data_set = [
        PhTestConst.SRC_CODES_PH_FILES_PATHS,
        PhTestConst.SRC_CODE_TEST_DIR_PATH,
        PhUtil.path_current_folder,
        path_current_script,
        name_current_script,
    ]
    # TODO: Check if below approach (loop handling) can be made generic
    for _input_data_set in input_data_set:
        _input_data_set_items = PhUtil.normalise_list(_input_data_set)
        if isinstance(_input_data_set_items, list):
            for input_data in _input_data_set_items:
                print(f'{input_data}: {PhUtil.last_modification_time(file_or_folder_path=input_data)}')
        else:
            print(
                f'{_input_data_set_items}: {PhUtil.last_modification_time(file_or_folder_path=_input_data_set_items)}')
        print(PhConstants.SEPERATOR_TWO_DATA_SET)


def test_generate_test_data():
    PhUtil.print_heading()
    #
    sep = PhConstants.SEPERATOR_MULTI_LINE
    input_data_set = [
        0,
        1,
        -5,
        5,
        226,
        227,
        228,
        229,
        14765,
    ]
    for input_data in input_data_set:
        PhUtil.print_heading(f'require_length={input_data}')
        test_data = PhUtil.generate_test_data(require_length=input_data)
        PhUtil.get_key_value_pair(key='Test Data', value=test_data, sep=sep, print_also=True)
        PhUtil.get_key_value_pair(key='Test Data', value=test_data, print_also=True, length_needed=True)


def test_generate_filters():
    PhUtil.print_heading()
    tool_pool = \
        [
            PhApps.NOTEPAD_PLUS_PLUS,
            PhApps.BEYOND_COMPARE,
            'Unknown Tool',
        ]
    for tool in tool_pool:
        PhUtil.print_output(heading=tool, output_data=PhMisc.generate_filters(tool))


def test_functions(ph_time):
    """

    :param ph_time:
    :return:
    """
    test_temp()
    test_open_bugs()
    ## Keep on the 3rd Number
    ##
    test_version()
    test_parse_config()
    test_expired_attr()
    test_doc_string()
    test_get_help_for_param()
    test_chars_to_utf8()
    test_get_git_info()
    test_get_time_stamp_file_name()
    test_to_file()
    test_misc()
    test_list()
    test_heading()
    test_python_friendly_name()
    test_remarks_append_post()
    test_remarks_append_pre()
    test_print_all_installed_distributions()
    test_print_modules()
    test_print_iter()
    test_obj_list()
    test_get_classes_list()
    test_generalise_list()
    test_hash()
    test_zipfile()
    test_traverse_it()
    test_eid()
    test_luhn()
    test_normalise_list()
    test_handle_dirs()
    test_generate_test_data()
    test_trim_white_spaces_in_str()
    test_last_modification_time()
    test_generate_filters()
    ##
    ## Keep on last
    test_time_delay(ph_time)


def main():
    """

    :return:
    """
    """
    Time Object
    """
    ph_time = PhTime()
    ph_time.start()
    """
    Process
    """
    # _unit_test_mode = True
    if _unit_test_mode:
        """
        Temp Test Cases
        """
        test_temp()
    else:
        """
        Regular Test Cases
        """
        test_functions(ph_time)
    """
    Wrap up 
    """
    ph_time.stop()
    ph_time.print()
    PhUtil.print_done()


if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        # separator
        msg = [
            '',
            '=' * 30,
            'ERROR DETECTED',
            '=' * 30,
            str(e),
            '',
        ]
        sys.stderr.write('\n'.join(msg))
        # 2. Re-raise the error to let the full traceback print to stderr
        raise
